<?php

use CKPL\Pay\Storage\AbstractStorage;
use CKPL\Pay\Storage\StorageInterface;
use PrestaShop\Module\ConotoxiaPay\Definitions;

/**
 * Class PS_Gateway_Conotoxia_Pay_Storage
 */
class PS_Gateway_Conotoxia_Pay_Storage extends AbstractStorage
{
    /**
     * @type PrestaShop\PrestaShop\Adapter\Configuration
     */
    private $configuration;

    /**
     * PS_Gateway_Conotoxia_Pay_Storage constructor.
     * @param $configuration PrestaShop\PrestaShop\Adapter\Configuration $configuration
     * @throws \PrestaShop\PrestaShop\Core\Domain\Shop\Exception\ShopException
     */
    public function __construct($configuration)
    {
        $this->configuration = $configuration;
        $this->items = [];
        $this->load();
    }

    /**
     * @param string $key
     *
     * @return bool
     */
    public function hasItem(string $key): bool
    {
        return \array_key_exists($key, $this->items);
    }

    /**
     * @param string $key
     * @param mixed $value
     *
     * @return void
     * @throws Exception
     */
    public function setItem(string $key, $value): void
    {
        if ($key === static::TOKEN || $key === static::PAYMENT_SERVICE_PUBLIC_KEYS) {
            $this->configuration->set(Definitions::SETTING_PREFIX . strtoupper($key), \json_encode($value, JSON_PRESERVE_ZERO_FRACTION | JSON_UNESCAPED_SLASHES));
            $this->items[$key] = $value;
        } else {
            $this->configuration->set(Definitions::SETTING_PREFIX . strtoupper($key), $value);
            $this->items[$key] = (string)$value;
        }
    }

    /**
     * @return void
     */
    protected function load(): void
    {
        foreach ([static::TOKEN, static::PAYMENT_SERVICE_PUBLIC_KEYS, static::PUBLIC_KEY_ID, static::PUBLIC_KEY_CHECKSUM] as $item) {
            $value = $this->configuration->get(strtoupper(Definitions::SETTING_PREFIX . $item));
            if (!empty($value)) {
                if ($item === static::TOKEN || $item === static::PAYMENT_SERVICE_PUBLIC_KEYS) {
                    $this->items[$item] = \json_decode($value, true);
                } else {
                    $this->items[$item] = $value;
                }
            }
        }
    }

    /**
     * @return void
     * @throws Exception
     */
    public function clear(): void
    {
        $this->configuration->set(Definitions::SETTING_PREFIX . strtoupper(StorageInterface::TOKEN), '');
        $this->configuration->set(Definitions::SETTING_PREFIX . strtoupper(StorageInterface::PUBLIC_KEY_CHECKSUM), '');
        $this->configuration->set(Definitions::SETTING_PREFIX . strtoupper(StorageInterface::PAYMENT_SERVICE_PUBLIC_KEYS), '');
        $this->configuration->set(Definitions::SETTING_PREFIX . strtoupper(StorageInterface::PUBLIC_KEY_ID), '');
    }
}
